package com.bweb.controller;
import com.alibaba.fastjson.JSONObject;
import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.common.utils.*;
import com.hfkj.config.CommonSysConst;

import com.hfkj.entity.BsMerPayConfig;
import com.hfkj.entity.BsOrder;
import com.hfkj.entity.BsUserCard;
import com.hfkj.jd.JdService;
import com.hfkj.mer.BsMerPayConfigService;
import com.hfkj.model.ResponseData;

import com.hfkj.model.order.OrderModel;
import com.hfkj.qianzhu.CinemaService;
import com.hfkj.service.card.BsUserCardService;
import com.hfkj.service.hlt.HuiLianTongUnionCardService;
import com.hfkj.service.order.BsOrderService;
import com.hfkj.service.order.OrderPayService;
import com.hfkj.service.pay.huift.TradeService;
import com.hfkj.sysenum.mer.PayPlatformEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;


@Controller
@RequestMapping(value="/test")
@Api(value="测试")
public class TestController {
    private static Logger log = LoggerFactory.getLogger(TestController.class);
    @Resource
    private TradeService huiftTradeService;
    @Resource
    private BsOrderService orderService;
    @Resource
    private BsMerPayConfigService merPayConfigService;
    @Resource
    private OrderPayService orderPayService;
    @Resource
    private BsUserCardService userCardService;
    @RequestMapping(value="/thirdPayResult",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询V家园订单交易状态")
    public ResponseData thirdPayResult(@RequestParam(value = "preOrderNo" , required = true) String preOrderNo,
                                       @RequestParam(value = "shopCode" , required = true) String shopCode,
                                       @RequestParam(value = "signSecret" , required = true) String signSecret) {
        try {

            return ResponseMsgUtil.success(huiftTradeService.thirdPayResult(preOrderNo,shopCode,signSecret));

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/haiOilLogin",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "嗨加油登陆参数")
    public ResponseData haiOilLogin(@RequestBody JSONObject body) {
        try {

            if (body == null|| StringUtils.isBlank(body.getString("phone"))) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            Map<String , Object> map = new HashMap<>();
            map.put("appid" , CommonSysConst.getSysConfig().getHaiOilAppid());
            map.put("phone" , body.getString("phone"));
            map.put("sign" , SignatureUtil.createSign(map , CommonSysConst.getSysConfig().getHaiOilAppSecret()));


            return ResponseMsgUtil.success(map);

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/refundMovieOrder",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "refundMovieOrder")
    public ResponseData refundMovieOrder(@RequestParam(value = "orderNo" , required = true) String orderNo,
                                         @RequestParam(value = "payRealPrice" , required = true) BigDecimal payRealPrice,
                                         @RequestParam(value = "cardNo" , required = true) String cardNo) {
        try {


            // 工会卡支付
            JSONObject consumption = HuiLianTongUnionCardService.consumption(orderNo, cardNo, payRealPrice, "HiSen_phg_goods", "11101608", "", "241001130753599939订单扣款");
            if (!consumption.getString("respCode").equals("0000")) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, consumption.getString("respMessage"));
            }
            // 响应参数
            JSONObject consumptionResult = HuiLianTongUnionCardService.resolveResponse(consumption.getString("data"));
            if (consumptionResult.getBoolean("success") != true) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, consumptionResult.getString("message"));
            }

            return ResponseMsgUtil.success(consumptionResult);

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/getAccessToken",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "getAccessToken")
    public ResponseData getAccessToken() {
        try {

            return ResponseMsgUtil.success(JdService.getAccessToken());

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

}
