package com.order.controller.business;

import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.common.security.SessionObject;
import com.hfkj.common.security.UserCenter;
import com.hfkj.common.utils.ResponseMsgUtil;
import com.hfkj.entity.BsOrderMeituan;
import com.hfkj.entity.SecConfig;
import com.hfkj.model.ResponseData;
import com.hfkj.model.SecUserSessionObject;
import com.hfkj.service.SecConfigService;
import com.hfkj.service.goods.BsOrderMeiTuanService;
import com.hfkj.service.goods.impl.BsOrderMeiTuanServiceImpl;
import com.hfkj.sysenum.SecUserObjectTypeEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.HashMap;
import java.util.Map;

@Controller
@RequestMapping(value="/orderMeiTuan")
@Api(value="美团订单管理")
public class BsOrderMeiTuanController {

    @Resource
    private SecConfigService secConfigService;

    @Resource
    private BsOrderMeiTuanService orderMeiTuanService;

    @Resource
    private UserCenter userCenter;


    private static final Logger log = LoggerFactory.getLogger(BsOrderGoodsController.class);
    @Autowired
    private BsOrderMeiTuanServiceImpl bsOrderMeiTuanService;

    @RequestMapping(value="/servicePrice",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询服务费")
    public ResponseData servicePrice(@RequestParam(value = "platformCode" , required = true) String platformCode,
                                     @RequestParam(value = "tradeNo" , required = true) String tradeNo,
                                     @RequestParam(value = "price" , required = true) BigDecimal price
    ) {
        try {
            // 查询配置的平台服务费
            SecConfig config = secConfigService.queryDetail(platformCode);
            BigDecimal servicePrice;

            if (config == null) {
                log.error("orderMeiTuan -> servicePrice() error!");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未找到相信息");
            }

            BsOrderMeituan orderMeiTuan = orderMeiTuanService.findOrderMeiTuanByTradeNo(tradeNo);

            servicePrice = price.multiply(new BigDecimal(config.getCodeValue())).setScale(2 , RoundingMode.HALF_UP);


            JSONObject object = new JSONObject();

            object.put("servicePrice" , servicePrice);
            object.put("goodsName" , orderMeiTuan.getGoodsName());
            object.put("tradeAmount" , orderMeiTuan.getTradeAmount());

            return ResponseMsgUtil.success(object);

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }


    @RequestMapping(value="/getListMeiTuan",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询列表")
    public ResponseData getListMeiTuan(
                                     @RequestParam(value = "orderNo" , required = false) String orderNo,
                                     @RequestParam(value = "childOrderNo" , required = false) String childOrderNo,
                                     @RequestParam(value = "userPhone" , required = false) String userPhone,
                                     @RequestParam(value = "goodsName" , required = false) String goodsName,
                                     @RequestParam(value = "tradeNo" , required = false) String tradeNo,
                                     @RequestParam(value = "payType" , required = false) Integer payType,
                                     @RequestParam(value = "status" , required = false) Integer status,
                                     @RequestParam(value = "createTimeS" , required = false) Long createTimeS,
                                     @RequestParam(value = "createTimeE" , required = false) Long createTimeE,
                                     @RequestParam(value = "finishTimeS" , required = false) Long finishTimeS,
                                     @RequestParam(value = "finishTimeE" , required = false) Long finishTimeE,
                                     @RequestParam(value = "pageNum" , required = true) Integer pageNum,
                                     @RequestParam(value = "pageSize" , required = true) Integer pageSize) {
        try {

            Map<String , Object> map = new HashMap<>();

            map.put("orderNo", orderNo);
            map.put("childOrderNo", childOrderNo);
            map.put("userPhone", userPhone);
            map.put("tradeNo", tradeNo);
            map.put("goodsName", goodsName);
            map.put("payType", payType);
            map.put("createTimeS", createTimeS);
            map.put("createTimeE", createTimeE);
            map.put("finishTimeS", finishTimeS);
            map.put("finishTimeE", finishTimeE);
            map.put("status", status);

            PageHelper.startPage(pageNum, pageSize);
            return ResponseMsgUtil.success(new PageInfo<>(bsOrderMeiTuanService.getOrderList(map)));

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }
}
