package com.order.controller.business;

import com.alibaba.excel.EasyExcel;
import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.common.security.SessionObject;
import com.hfkj.common.security.UserCenter;
import com.hfkj.common.utils.DateUtil;
import com.hfkj.common.utils.ResponseMsgUtil;
import com.hfkj.config.CommonSysConst;
import com.hfkj.entity.BsOrderGoods;
import com.hfkj.entity.FileRecords;
import com.hfkj.model.ResponseData;
import com.hfkj.model.SecUserSessionObject;

import com.hfkj.model.order.orderExport.MerOrderSettlementDetailModel;
import com.hfkj.model.order.orderExport.OrderGoodsExportModel;
import com.hfkj.model.order.orderExport.OrderSettlementModel;
import com.hfkj.model.order.orderExport.OrderSettlementSumModel;
import com.hfkj.service.file.FileRecordsService;
import com.hfkj.service.order.OrderSettlementService;
import com.hfkj.sysenum.SecUserObjectTypeEnum;
import com.hfkj.sysenum.fileRecords.FileRecordsEnum;
import com.hfkj.sysenum.order.OrderChildStatusEnum;
import com.hfkj.sysenum.order.OrderPayChannelEnum;
import com.hfkj.sysenum.order.OrderPayTypeEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.beanutils.BeanUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

@Controller
@RequestMapping(value="/settlement")
@Api(value="订单统计业务")
public class OrderSettlementController {

    private static final Logger log = LoggerFactory.getLogger(OrderSettlementController.class);

    @Resource
    private UserCenter userCenter;
    @Resource
    private OrderSettlementService orderSettlementService;
    @Resource
    private FileRecordsService fileRecordsService;


    @RequestMapping(value="/settlementDetail",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "结算详情")
    public ResponseData settlementDetail(@RequestParam(value = "platform" , required = false) String platform,
                                         @RequestParam(value = "merId" , required = false) Long merId,
                                         @RequestParam(value = "payType" , required = false) Long payType,
                                         @RequestParam(value = "finishTimeS" , required = false) Long finishTimeS,
                                         @RequestParam(value = "finishTimeE" , required = false) Long finishTimeE, HttpServletRequest request) {
        try {
            Map<String,Object> param = new HashMap<>();
            SessionObject sessionObject = userCenter.getSessionObject(request);
            SecUserSessionObject userModel = (SecUserSessionObject) sessionObject.getObject();
            if (userModel.getAccount().getObjectType().equals(SecUserObjectTypeEnum.type1.getCode())) {
                param.put("merId", merId);
            }
            if (SecUserObjectTypeEnum.type2.getCode() == userModel.getAccount().getObjectType()) {
                param.put("merId", userModel.getAccount().getObjectId());
            }
            param.put("platform", platform);
            param.put("payType", payType);
            param.put("finishTimeS", finishTimeS);
            param.put("finishTimeE", finishTimeE);

            return ResponseMsgUtil.success(orderSettlementService.getDetail(param));

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/exportSettlementDetail",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "导出结算详情")
    public ResponseData exportSettlementDetail(@RequestParam(value = "platform" , required = false) String platform,
                                               @RequestParam(value = "merId" , required = false) Long merId,
                                               @RequestParam(value = "payType" , required = false) Long payType,
                                               @RequestParam(value = "finishTimeS" , required = false) Long finishTimeS,
                                               @RequestParam(value = "finishTimeE" , required = false) Long finishTimeE, HttpServletRequest request) {
        // 插入下载记录
        FileRecords fileRecords = new FileRecords();
        try {

            Map<String,Object> param = new HashMap<>();
            SessionObject sessionObject = userCenter.getSessionObject(request);
            SecUserSessionObject userModel = (SecUserSessionObject) sessionObject.getObject();
            if (userModel.getAccount().getObjectType().equals(SecUserObjectTypeEnum.type1.getCode())) {
                param.put("merId", merId);
            }
            if (SecUserObjectTypeEnum.type2.getCode() == userModel.getAccount().getObjectType()) {
                param.put("merId", userModel.getAccount().getObjectId());
            }
            param.put("platform", platform);
            param.put("payType", payType);
            param.put("finishTimeS", finishTimeS);
            param.put("finishTimeE", finishTimeE);

            String dataSting = DateUtil.date2String(new Date(), DateUtil.YMDHMS);

            String fileUrl = CommonSysConst.getSysConfig().getFileUrl();
            String filePath = "/temporary/" + userModel.getAccount().getId() + "/" ;
            String pathName =  filePath + FileRecordsEnum.status10.getName() +  "交易数据导出"  + dataSting +".xlsx";

            if (fileRecordsService.findExported(param.toString() , userModel.getAccount().getId())) {
                throw ErrorHelp.genException(SysCode.System , ErrorCode.COMMON_ERROR , "相同导出内容请勿重复点击！");
            }

            fileRecords.setCreateTime(new Date());
            fileRecords.setBusinessType(FileRecordsEnum.status10.getCode());
            fileRecords.setDownloadConditions(param.toString());
            fileRecords.setOpId(userModel.getAccount().getId());
            fileRecords.setOpName(userModel.getAccount().getUserName());
            fileRecords.setStatus(2);
            fileRecords.setDownloadStatus(2);
            fileRecords.setFileUrl(CommonSysConst.getSysConfig().getDomainName() + "/filesystem" + pathName);
            fileRecords.setType("2");
            fileRecords.setTitle(FileRecordsEnum.status10.getName() +  "交易数据导出" + dataSting  + ".xls");
            fileRecordsService.create(fileRecords);

            // 创建线程
            ThreadPoolExecutor executor = new ThreadPoolExecutor(
                    2, // 核心线程数
                    5, // 最大线程数
                    1, // 线程空闲时间
                    TimeUnit.MINUTES, // 时间单位
                    new ArrayBlockingQueue<>(5), // 任务队列
                    new ThreadPoolExecutor.CallerRunsPolicy() // 拒绝策略
            );

            // 提交任务给线程池
            executor.execute(() -> {

                try {

                    List<MerOrderSettlementDetailModel> list = orderSettlementService.getDetail(param);

                    File file = new File(fileUrl + filePath);
                    if (!file.exists()) {
                        file.mkdirs();
                    }

                    EasyExcel.write(fileUrl+pathName)
                            .head(MerOrderSettlementDetailModel.class)
                            .sheet(fileRecords.getTitle())
                            .doWrite(list);

                    fileRecords.setStatus(1);
                    fileRecords.setUpdateTime(new Date());
                    fileRecordsService.update(fileRecords);
                } catch (Exception e) {
                    fileRecords.setStatus(3);
                    fileRecords.setUpdateTime(new Date());
                    fileRecords.setExt1(e.toString());
                    fileRecordsService.update(fileRecords);
                    log.error("error!",e);
                }
            });

            // 关闭线程池
            executor.shutdown();
            return ResponseMsgUtil.success(fileRecords);

        } catch (Exception e) {
            fileRecords.setStatus(3);
            fileRecords.setUpdateTime(new Date());
            fileRecordsService.update(fileRecords);
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/getSettlement",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "结算详情")
    public ResponseData getSettlement(@RequestParam(value = "platform" , required = false) String platform,
                                         @RequestParam(value = "merId" , required = false) Long merId,
                                         @RequestParam(value = "payType" , required = false) Long payType,
                                         @RequestParam(value = "finishTimeS" , required = false) Long finishTimeS,
                                         @RequestParam(value = "finishTimeE" , required = false) Long finishTimeE, HttpServletRequest request) {
        try {
            Map<String,Object> param = new HashMap<>();
            SessionObject sessionObject = userCenter.getSessionObject(request);
            SecUserSessionObject userModel = (SecUserSessionObject) sessionObject.getObject();
            if (userModel.getAccount().getObjectType().equals(SecUserObjectTypeEnum.type1.getCode())) {
                param.put("merId", merId);
            }
            if (SecUserObjectTypeEnum.type2.getCode() == userModel.getAccount().getObjectType()) {
                param.put("merId", userModel.getAccount().getObjectId());
            }
            param.put("platform", platform);
            param.put("payType", payType);
            param.put("finishTimeS", finishTimeS);
            param.put("finishTimeE", finishTimeE);

            return ResponseMsgUtil.success(orderSettlementService.getSettlement(param));

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/exportSettlement",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "导出结算")
    public ResponseData exportSettlement(@RequestParam(value = "platform" , required = false) String platform,
                                               @RequestParam(value = "merId" , required = false) Long merId,
                                               @RequestParam(value = "payType" , required = false) Long payType,
                                               @RequestParam(value = "finishTimeS" , required = false) Long finishTimeS,
                                               @RequestParam(value = "finishTimeE" , required = false) Long finishTimeE, HttpServletRequest request) {
        // 插入下载记录
        FileRecords fileRecords = new FileRecords();
        try {

            Map<String,Object> param = new HashMap<>();
            SessionObject sessionObject = userCenter.getSessionObject(request);
            SecUserSessionObject userModel = (SecUserSessionObject) sessionObject.getObject();
            if (userModel.getAccount().getObjectType().equals(SecUserObjectTypeEnum.type1.getCode())) {
                param.put("merId", merId);
            }
            if (SecUserObjectTypeEnum.type2.getCode() == userModel.getAccount().getObjectType()) {
                param.put("merId", userModel.getAccount().getObjectId());
            }
            param.put("platform", platform);
            param.put("payType", payType);
            param.put("finishTimeS", finishTimeS);
            param.put("finishTimeE", finishTimeE);

            String dataSting = DateUtil.date2String(new Date(), DateUtil.YMDHMS);

            String fileUrl = CommonSysConst.getSysConfig().getFileUrl();
            String filePath = "/temporary/" + userModel.getAccount().getId() + "/" ;
            String pathName =  filePath + FileRecordsEnum.status11.getName() +  "交易数据导出"  + dataSting +".xlsx";

            if (fileRecordsService.findExported(param.toString() , userModel.getAccount().getId())) {
                throw ErrorHelp.genException(SysCode.System , ErrorCode.COMMON_ERROR , "相同导出内容请勿重复点击！");
            }

            fileRecords.setCreateTime(new Date());
            fileRecords.setBusinessType(FileRecordsEnum.status11.getCode());
            fileRecords.setDownloadConditions(param.toString());
            fileRecords.setOpId(userModel.getAccount().getId());
            fileRecords.setOpName(userModel.getAccount().getUserName());
            fileRecords.setStatus(2);
            fileRecords.setDownloadStatus(2);
            fileRecords.setFileUrl(CommonSysConst.getSysConfig().getDomainName() + "/filesystem" + pathName);
            fileRecords.setType("2");
            fileRecords.setTitle(FileRecordsEnum.status11.getName() +  "交易数据导出" + dataSting  + ".xls");
            fileRecordsService.create(fileRecords);

            // 创建线程
            ThreadPoolExecutor executor = new ThreadPoolExecutor(
                    2, // 核心线程数
                    5, // 最大线程数
                    1, // 线程空闲时间
                    TimeUnit.MINUTES, // 时间单位
                    new ArrayBlockingQueue<>(5), // 任务队列
                    new ThreadPoolExecutor.CallerRunsPolicy() // 拒绝策略
            );

            // 提交任务给线程池
            executor.execute(() -> {

                try {

                    List<OrderSettlementModel> list = orderSettlementService.getSettlement(param);

                    File file = new File(fileUrl + filePath);
                    if (!file.exists()) {
                        file.mkdirs();
                    }

                    EasyExcel.write(fileUrl+pathName)
                            .head(OrderSettlementModel.class)
                            .sheet(fileRecords.getTitle())
                            .doWrite(list);

                    fileRecords.setStatus(1);
                    fileRecords.setUpdateTime(new Date());
                    fileRecordsService.update(fileRecords);
                } catch (Exception e) {
                    fileRecords.setStatus(3);
                    fileRecords.setUpdateTime(new Date());
                    fileRecords.setExt1(e.toString());
                    fileRecordsService.update(fileRecords);
                    log.error("error!",e);
                }
            });

            // 关闭线程池
            executor.shutdown();
            return ResponseMsgUtil.success(fileRecords);

        } catch (Exception e) {
            fileRecords.setStatus(3);
            fileRecords.setUpdateTime(new Date());
            fileRecordsService.update(fileRecords);
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/getSum",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "结算总和")
    public ResponseData getSum(@RequestParam(value = "platform" , required = false) String platform,
                                      @RequestParam(value = "merId" , required = false) Long merId,
                                      @RequestParam(value = "payType" , required = false) Long payType,
                                      @RequestParam(value = "finishTimeS" , required = false) Long finishTimeS,
                                      @RequestParam(value = "finishTimeE" , required = false) Long finishTimeE, HttpServletRequest request) {
        try {
            Map<String,Object> param = new HashMap<>();
            SessionObject sessionObject = userCenter.getSessionObject(request);
            SecUserSessionObject userModel = (SecUserSessionObject) sessionObject.getObject();
            if (userModel.getAccount().getObjectType().equals(SecUserObjectTypeEnum.type1.getCode())) {
                param.put("merId", merId);
            }
            if (SecUserObjectTypeEnum.type2.getCode() == userModel.getAccount().getObjectType()) {
                param.put("merId", userModel.getAccount().getObjectId());
            }
            param.put("platform", platform);
            param.put("payType", payType);
            param.put("finishTimeS", finishTimeS);
            param.put("finishTimeE", finishTimeE);

            return ResponseMsgUtil.success(orderSettlementService.getSum(param));

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/exportSettlementSum",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "导出结算总和")
    public ResponseData exportSettlementSum(@RequestParam(value = "platform" , required = false) String platform,
                                         @RequestParam(value = "merId" , required = false) Long merId,
                                         @RequestParam(value = "payType" , required = false) Long payType,
                                         @RequestParam(value = "finishTimeS" , required = false) Long finishTimeS,
                                         @RequestParam(value = "finishTimeE" , required = false) Long finishTimeE, HttpServletRequest request) {
        // 插入下载记录
        FileRecords fileRecords = new FileRecords();
        try {

            Map<String,Object> param = new HashMap<>();
            SessionObject sessionObject = userCenter.getSessionObject(request);
            SecUserSessionObject userModel = (SecUserSessionObject) sessionObject.getObject();
            if (userModel.getAccount().getObjectType().equals(SecUserObjectTypeEnum.type1.getCode())) {
                param.put("merId", merId);
            }
            if (SecUserObjectTypeEnum.type2.getCode() == userModel.getAccount().getObjectType()) {
                param.put("merId", userModel.getAccount().getObjectId());
            }
            param.put("platform", platform);
            param.put("payType", payType);
            param.put("finishTimeS", finishTimeS);
            param.put("finishTimeE", finishTimeE);

            String dataSting = DateUtil.date2String(new Date(), DateUtil.YMDHMS);

            String fileUrl = CommonSysConst.getSysConfig().getFileUrl();
            String filePath = "/temporary/" + userModel.getAccount().getId() + "/" ;
            String pathName =  filePath + FileRecordsEnum.status12.getName() +  "交易数据导出"  + dataSting +".xlsx";

            if (fileRecordsService.findExported(param.toString() , userModel.getAccount().getId())) {
                throw ErrorHelp.genException(SysCode.System , ErrorCode.COMMON_ERROR , "相同导出内容请勿重复点击！");
            }

            fileRecords.setCreateTime(new Date());
            fileRecords.setBusinessType(FileRecordsEnum.status12.getCode());
            fileRecords.setDownloadConditions(param.toString());
            fileRecords.setOpId(userModel.getAccount().getId());
            fileRecords.setOpName(userModel.getAccount().getUserName());
            fileRecords.setStatus(2);
            fileRecords.setDownloadStatus(2);
            fileRecords.setFileUrl(CommonSysConst.getSysConfig().getDomainName() + "/filesystem" + pathName);
            fileRecords.setType("2");
            fileRecords.setTitle(FileRecordsEnum.status12.getName() +  "交易数据导出" + dataSting  + ".xls");
            fileRecordsService.create(fileRecords);

            // 创建线程
            ThreadPoolExecutor executor = new ThreadPoolExecutor(
                    2, // 核心线程数
                    5, // 最大线程数
                    1, // 线程空闲时间
                    TimeUnit.MINUTES, // 时间单位
                    new ArrayBlockingQueue<>(5), // 任务队列
                    new ThreadPoolExecutor.CallerRunsPolicy() // 拒绝策略
            );

            // 提交任务给线程池
            executor.execute(() -> {

                try {

                    List<OrderSettlementSumModel> list = orderSettlementService.getSum(param);

                    File file = new File(fileUrl + filePath);
                    if (!file.exists()) {
                        file.mkdirs();
                    }

                    EasyExcel.write(fileUrl+pathName)
                            .head(OrderSettlementSumModel.class)
                            .sheet(fileRecords.getTitle())
                            .doWrite(list);

                    fileRecords.setStatus(1);
                    fileRecords.setUpdateTime(new Date());
                    fileRecordsService.update(fileRecords);
                } catch (Exception e) {
                    fileRecords.setStatus(3);
                    fileRecords.setUpdateTime(new Date());
                    fileRecords.setExt1(e.toString());
                    fileRecordsService.update(fileRecords);
                    log.error("error!",e);
                }
            });

            // 关闭线程池
            executor.shutdown();
            return ResponseMsgUtil.success(fileRecords);

        } catch (Exception e) {
            fileRecords.setStatus(3);
            fileRecords.setUpdateTime(new Date());
            fileRecordsService.update(fileRecords);
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }
}
