package com.hfkj.service.order.impl;

import com.alibaba.fastjson.JSONObject;
import com.hfkj.common.utils.DateUtil;
import com.hfkj.entity.*;
import com.hfkj.model.order.OrderCouponModel;
import com.hfkj.model.order.orderExport.MerOrderSettlementDetailModel;
import com.hfkj.model.order.orderExport.OrderSettlementModel;
import com.hfkj.model.order.orderExport.OrderSettlementSumModel;
import com.hfkj.service.BsMerService;
import com.hfkj.service.coupon.BsOrderCouponService;
import com.hfkj.service.goods.*;
import com.hfkj.service.order.OrderSettlementService;
import com.hfkj.sysenum.UserLoginPlatform;
import com.hfkj.sysenum.order.OrderPayTypeEnum;
import org.apache.commons.collections4.MapUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Service("orderSettlementService")
public class OrderSettlementServiceImpl implements OrderSettlementService {

    @Resource
    private BsOrderCouponService orderCouponService;
    @Resource
    private BsOrderGoodsService goodsService;
    @Resource
    private BsOrderCinemaService cinemaService;
    @Resource
    private BsOrderStarbucksService starbucksService;
    @Resource
    private BsOrderMeiTuanService meiTuanService;
    @Resource
    private BsOrderMemberService memberService;
    @Resource
    private BsMerService bsMerService;
    @Override
    public List<MerOrderSettlementDetailModel> getDetail(Map<String, Object> map) {
        return settlementData(map);
    }

    @Override
    public List<OrderSettlementModel> getSettlement(Map<String, Object> map) {

        List<MerOrderSettlementDetailModel> list = settlementData(map);

        Map<String, List<MerOrderSettlementDetailModel>> merList = list.stream().collect(Collectors.groupingBy(MerOrderSettlementDetailModel:: getMerName));

        List<OrderSettlementModel> settlementModels = new ArrayList<>();
        OrderSettlementModel settlementModel;

        for (String key : merList.keySet()) {
            Map<String, List<MerOrderSettlementDetailModel>> detailList = merList.get(key).stream().collect(Collectors.groupingBy(MerOrderSettlementDetailModel:: getBusinessType));
            for (String key1 : detailList.keySet()) {
                Map<String, List<MerOrderSettlementDetailModel>> listMap = detailList.get(key1).stream().collect(Collectors.groupingBy(MerOrderSettlementDetailModel:: getPayType));
                for (String key2 : listMap.keySet()) {
                    settlementModel = new OrderSettlementModel();
                    if (map.get("finishTimeS") == null) {
                        settlementModel.setTime("全时间段");
                    } else {
                        settlementModel.setTime(DateUtil.date2String(DateUtil.long2Date(MapUtils.getLong(map , "finishTimeS")) , DateUtil.Y_M_D) + "-" + DateUtil.date2String(DateUtil.long2Date(MapUtils.getLong(map , "finishTimeE")) , DateUtil.Y_M_D));
                    }
                    settlementModel.setPayType(key2);
                    settlementModel.setBusinessType(key1);
                    settlementModel.setMerName(key);
                    BigDecimal reduce = listMap.get(key2).stream().map(MerOrderSettlementDetailModel::getPayRealPrice).reduce(BigDecimal.ZERO, BigDecimal::add);
                    settlementModel.setPayRealPrice(reduce);
                    settlementModels.add(settlementModel);
                }


            }

        }
        return settlementModels;
    }


    @Override
    public List<OrderSettlementSumModel> getSum(Map<String, Object> map) {

        List<MerOrderSettlementDetailModel> list = settlementData(map);

        List<OrderSettlementSumModel> sumModels = new ArrayList<>();
        OrderSettlementSumModel sumModel;

        Map<String, List<MerOrderSettlementDetailModel>> detailList = list.stream().collect(Collectors.groupingBy(MerOrderSettlementDetailModel:: getBusinessType));
        for (String key1 : detailList.keySet()) {
            Map<String, List<MerOrderSettlementDetailModel>> listMap = detailList.get(key1).stream().collect(Collectors.groupingBy(MerOrderSettlementDetailModel:: getPayType));
            for (String key2 : listMap.keySet()) {
                sumModel = new OrderSettlementSumModel();
                if (map.get("finishTimeS") == null) {
                    sumModel.setTime("全时间段");
                } else {
                    sumModel.setTime(DateUtil.date2String(DateUtil.long2Date(MapUtils.getLong(map , "finishTimeS")) , DateUtil.Y_M_D) + "-" + DateUtil.date2String(DateUtil.long2Date(MapUtils.getLong(map , "finishTimeE")) , DateUtil.Y_M_D));
                }
                sumModel.setPayType(key2);
                sumModel.setBusinessType(key1);
                BigDecimal reduce = listMap.get(key2).stream().map(MerOrderSettlementDetailModel::getPayRealPrice).reduce(BigDecimal.ZERO, BigDecimal::add);
                sumModel.setPayRealPrice(reduce);
                sumModels.add(sumModel);
            }


        }
        return sumModels;
    }

    // 查询结算订单详情
    private List<MerOrderSettlementDetailModel> settlementData(Map<String, Object> map) {
        List<MerOrderSettlementDetailModel> list = new ArrayList<>();
        // 卡券订单整合
        list = couponOrder(map , list);
        // 查询实物订单
        list = goodsOrder(map , list);
        // 查询电影票订单
        list = cinemaOrder(map , list);
        // 查询星巴克
        list = starbucksOrder(map , list);
        // 查询美团订单
        list = meiTuanOrder(map , list);
        // 查询会员订单
        list = membersOrder(map , list);

        return  list;
    }

    // 实物订单整合
    private List<MerOrderSettlementDetailModel> goodsOrder(Map<String, Object> map, List<MerOrderSettlementDetailModel> list) {
        // 查询实物订单
        List<BsOrderGoods> orderGoodsList = goodsService.getListFinish(map);
        MerOrderSettlementDetailModel model;
        for (BsOrderGoods orderGoods : orderGoodsList) {
            model = new MerOrderSettlementDetailModel();
            model.setBusinessType("实物订单");
            model.setGoodsName(orderGoods.getGoodsName());
            model.setMerName(findMerName(orderGoods.getMerId()));
            if (orderGoods.getPlatformCode() != null) {
                model.setPlatform(UserLoginPlatform.getDataByType(orderGoods.getPlatformCode()).getName());
            }
            model.setOrderNo(orderGoods.getOrderNo());
            model.setChildOrderNo(orderGoods.getChildOrderNo());
            model.setBusiness(orderGoods.getLogisticsNo());
            if (orderGoods.getPayRealPrice() == null) {
                model.setPayRealPrice(new BigDecimal(0));
            } else {
                model.setPayRealPrice(orderGoods.getPayRealPrice());
            }

            if (orderGoods.getPayType() != null) {
                model.setPayType(OrderPayTypeEnum.getData(orderGoods.getPayType()).getName());
            } else {
                model.setPayType("未知");
            }
            if (orderGoods.getFinishTime() != null) {
                model.setFinishTime(DateUtil.date2String(orderGoods.getFinishTime() , DateUtil.Y_M_D_HMS));
            } else {
                model.setFinishTime(DateUtil.date2String(orderGoods.getUpdateTime() , DateUtil.Y_M_D_HMS));
            }

            list.add(model);
        }
        return list;
    }

    // 卡券订单整合
    private List<MerOrderSettlementDetailModel> couponOrder(Map<String, Object> map , List<MerOrderSettlementDetailModel> list) {
        // 查询卡券订单
        List<BsOrderCoupon> orderCouponList = orderCouponService.getOrderCouponList(map);
        Map<String , Object> mapCou = new HashMap<>();
        mapCou.put("couStatus", 2);
        // 查询卡券订单卡密
        List<OrderCouponModel> orderCouponModelList = orderCouponService.getOrderList(mapCou);

        MerOrderSettlementDetailModel model;
        for (BsOrderCoupon orderCoupon : orderCouponList) {
            String business = couponCode(orderCoupon.getChildOrderNo() , orderCouponModelList);
            if (business != null) {
                model = new MerOrderSettlementDetailModel();
                model.setGoodsName(orderCoupon.getGoodsName());
                model.setBusinessType("卡券订单");
                model.setMerName(findMerName(orderCoupon.getMerId()));
                if (orderCoupon.getPlatformCode() != null) {
                    model.setPlatform(UserLoginPlatform.getDataByType(orderCoupon.getPlatformCode()).getName());
                }
                model.setOrderNo(orderCoupon.getOrderNo());
                model.setChildOrderNo(orderCoupon.getChildOrderNo());
                model.setBusiness(business);
                if (orderCoupon.getPayRealPrice() == null) {
                    model.setPayRealPrice(new BigDecimal(0));
                } else {
                    model.setPayRealPrice(orderCoupon.getPayRealPrice());
                }
                if (orderCoupon.getPayType() != null) {
                    model.setPayType(OrderPayTypeEnum.getData(orderCoupon.getPayType()).getName());
                } else {
                    model.setPayType("未知");
                }
                model.setFinishTime(DateUtil.date2String(orderCoupon.getUpdateTime() , DateUtil.Y_M_D_HMS));
                list.add(model);
            }

        }

        return list;
    }
    // 整合电影票订单
    private List<MerOrderSettlementDetailModel> cinemaOrder(Map<String, Object> map, List<MerOrderSettlementDetailModel> list) {
        // 查询电影票订单
        List<BsOrderCinema> orderCinemaList = cinemaService.getListFinish(map);
        MerOrderSettlementDetailModel model;
        for (BsOrderCinema orderCinema : orderCinemaList) {
            model = new MerOrderSettlementDetailModel();
            model.setBusinessType("电影票订单");
            model.setGoodsName(orderCinema.getGoodsName());
            model.setMerName(findMerName(orderCinema.getMerId()));
            if (orderCinema.getPlatformCode() != null) {
                model.setPlatform(UserLoginPlatform.getDataByType(orderCinema.getPlatformCode()).getName());
            }
            model.setOrderNo(orderCinema.getOrderNo());
            model.setChildOrderNo(orderCinema.getChildOrderNo());
            if (orderCinema.getPayRealPrice() == null) {
                model.setPayRealPrice(new BigDecimal(0));
            } else {
                model.setPayRealPrice(orderCinema.getPayRealPrice());
            }
            if (orderCinema.getPayType() != null) {
                model.setPayType(OrderPayTypeEnum.getData(orderCinema.getPayType()).getName());
            } else {
                model.setPayType("未知");
            }

            model.setFinishTime(DateUtil.date2String(orderCinema.getCreateTime() , DateUtil.Y_M_D_HMS));
            list.add(model);
        }
        return list;
    }

    // 整合星巴克订单
    private List<MerOrderSettlementDetailModel> starbucksOrder(Map<String, Object> map, List<MerOrderSettlementDetailModel> list) {
        // 查询星巴克
        List<BsOrderStarbucks> orderStarbucksList = starbucksService.getListFinish(map);
        MerOrderSettlementDetailModel model;
        for (BsOrderStarbucks orderStarbucks : orderStarbucksList) {
            model = new MerOrderSettlementDetailModel();
            model.setBusinessType("星巴克订单");
            model.setGoodsName(orderStarbucks.getName());
            model.setMerName(findMerName(orderStarbucks.getMerId()));
            if (orderStarbucks.getPlatformCode() != null) {
                model.setPlatform(UserLoginPlatform.getDataByType(orderStarbucks.getPlatformCode()).getName());
            }
            model.setOrderNo(orderStarbucks.getOrderNo());
            model.setChildOrderNo(orderStarbucks.getChildOrderNo());
            if (orderStarbucks.getPrice() == null) {
                model.setPayRealPrice(new BigDecimal(0));
            } else {
                model.setPayRealPrice(orderStarbucks.getPrice());
            }
            if (orderStarbucks.getPayType() != null) {
                model.setPayType(OrderPayTypeEnum.getData(orderStarbucks.getPayType()).getName());
            } else {
                model.setPayType("未知");
            }
            model.setFinishTime(DateUtil.date2String(orderStarbucks.getCreateTime() , DateUtil.Y_M_D_HMS));
            list.add(model);
        }
        return list;
    }
    // 整合美团订单
    private List<MerOrderSettlementDetailModel> meiTuanOrder(Map<String, Object> map, List<MerOrderSettlementDetailModel> list) {
        // 查询美团订单
        List<BsOrderMeituan> orderMeituanList = meiTuanService.getOrderListFinish(map);
        MerOrderSettlementDetailModel model;
        for (BsOrderMeituan orderMeiTuan : orderMeituanList) {
            model = new MerOrderSettlementDetailModel();
            model.setBusinessType("美团订单");
            model.setBusiness(orderMeiTuan.getSqtBizOrderId());
            model.setGoodsName(orderMeiTuan.getGoodsName());
            model.setMerName(findMerName(orderMeiTuan.getMerId()));
            if (orderMeiTuan.getPlatformCode() != null) {
                model.setPlatform(UserLoginPlatform.getDataByType(orderMeiTuan.getPlatformCode()).getName());
            }
            model.setOrderNo(orderMeiTuan.getOrderNo());
            model.setChildOrderNo(orderMeiTuan.getChildOrderNo());
            if (orderMeiTuan.getPayRealPrice() == null) {
                model.setPayRealPrice(new BigDecimal(0));
            } else {
                model.setPayRealPrice(orderMeiTuan.getPayRealPrice());
            }
            if (orderMeiTuan.getPayType() != null) {
                model.setPayType(OrderPayTypeEnum.getData(orderMeiTuan.getPayType()).getName());
            } else {
                model.setPayType("未知");
            }
            if (orderMeiTuan.getFinishTime() != null) {
                model.setFinishTime(DateUtil.date2String(orderMeiTuan.getFinishTime() , DateUtil.Y_M_D_HMS));
            } else {
                model.setFinishTime(DateUtil.date2String(orderMeiTuan.getUpdateTime() , DateUtil.Y_M_D_HMS));
            }
            list.add(model);
        }
        return list;
    }
    // 整合会员订单
    private List<MerOrderSettlementDetailModel> membersOrder(Map<String, Object> map, List<MerOrderSettlementDetailModel> list) {
        // 查询会员订单
        List<BsOrderMember> orderMembersList = memberService.getListFinish(map);
        MerOrderSettlementDetailModel model;
        for (BsOrderMember orderMember : orderMembersList) {
            model = new MerOrderSettlementDetailModel();
            model.setBusinessType("会员订单");
            model.setGoodsName(orderMember.getGoodsName());
            model.setMerName(findMerName(orderMember.getMerId()));
            if (orderMember.getPlatformCode() != null) {
                model.setPlatform(UserLoginPlatform.getDataByType(orderMember.getPlatformCode()).getName());
            }
            model.setOrderNo(orderMember.getOrderNo());
            model.setChildOrderNo(orderMember.getChildOrderNo());
            if (orderMember.getPayRealPrice() == null) {
                model.setPayRealPrice(new BigDecimal(0));
            } else {
                model.setPayRealPrice(orderMember.getPayRealPrice());
            }
            if (orderMember.getPayType() != null) {
                model.setPayType(OrderPayTypeEnum.getData(orderMember.getPayType()).getName());
            } else {
                model.setPayType("未知");
            }
            if (orderMember.getFinishTime() != null) {
                model.setFinishTime(DateUtil.date2String(orderMember.getFinishTime() , DateUtil.Y_M_D_HMS));
            } else {
                model.setFinishTime(DateUtil.date2String(orderMember.getUpdateTime() , DateUtil.Y_M_D_HMS));
            }
            list.add(model);
        }
        return list;
    }


    private String findMerName(Long merId) {
        // 查询商户
        List<BsMer> bsMerList = bsMerService.getList(new HashMap<>());

        bsMerList = bsMerList.stream().filter(s -> s.getId().equals(merId)).collect(Collectors.toList());

        if (bsMerList.size() > 0) {
            return bsMerList.get(0).getMerName();
        }
        return merId.toString();
    }

    private String couponCode(String childOrderNo , List<OrderCouponModel> orderCouponModelList) {

        orderCouponModelList = orderCouponModelList.stream().filter(s -> s.getChildOrderNo().equals(childOrderNo)).collect(Collectors.toList());

        if (!orderCouponModelList.isEmpty()) {
            StringBuilder sb = new StringBuilder();

            for (OrderCouponModel orderCouponModel : orderCouponModelList) {
                sb.append(orderCouponModel.getCouNo()).append(",");
            }
            return sb.substring(0, sb.length() - 1);
        }
        return  null;
    }
}
